@@ -37,6 +37,7 @@ from sklearn.metrics import (
 )
 from sklearn.model_selection import train_test_split
 from sklearn.tree import export_graphviz
+from tabulate import tabulate
 
 from datto.CleanDataframe import CleanDataframe
 
@@ -277,6 +278,7 @@ class ModelResults:
         filename="shap_graph",
         path="../images/",
         multiclass=False,
+        y_test=None,
     ):
         """
         Displays graph of feature importances.
@@ -299,46 +301,55 @@ class ModelResults:
             'classification' or 'regression'
         filename: str
         multiclass: bool
+        y_test: pd.DataFrame
+            Only needed for multiclass models
         """
         if not os.path.exists(path):
             os.makedirs(path)
 
+        med = X_train.median().values.reshape((1, X_train.shape[1]))
+
+        # Runs too slow if X_test is huge, take a representative sample
+        if X_test.shape[0] > 1000:
+            X_test_sample = X_test.sample(1000)
+        else:
+            X_test_sample = X_test
+
         if multiclass:
-            f = lambda x: model.predict_proba(x)
+            lst_all_shap_values = []
+
+            for class_num in range(len(y_test.columns)):
+                f = lambda x: model.predict_proba(x)[class_num][:, 1]
+
+                explainer = shap.KernelExplainer(f, med)
+                shap_values = explainer.shap_values(X_test_sample)
+                lst_all_shap_values.append(shap_values)
+
+                class_name = y_test.columns[class_num]
+                print(f"SHAP Summary Plot for Class: {class_name}")
+
+                shap.summary_plot(shap_values, X_test_sample)
+                plt.tight_layout()
+                plt.savefig(f"{path}{class_name}_{filename}.png")
+                
+            return np.array(lst_all_shap_values)
+
         elif model_type.lower() == "classification":
             f = lambda x: model.predict_proba(x)[:, 1]
         else:
             f = lambda x: model.predict(x)
-        med = X_train.median().values.reshape((1, X_train.shape[1]))
+
         explainer = shap.KernelExplainer(f, med)
-
-        # Runs too slow if X_test is huge, take a representative sample
-        # Multiclass loads ultra slow, take an even smaller sample
-        if multiclass:
-            max_rows = 300
-        else:
-            max_rows = 1000
-
-        if X_test.shape[0] > max_rows:
-            X_test_sample = X_test.sample(max_rows)
-        else:
-            X_test_sample = X_test
-
         shap_values = explainer.shap_values(X_test_sample)
 
-        if multiclass:
-            for class_num in range(len(y_test.columns)):
-                class_name = y_test.columns[class_num]
-                print(f"SHAP Summary Plot for Class: {class_name}")
-                shap.summary_plot(shap_values[class_num], X_test_sample)
-                plt.tight_layout()
-                plt.savefig(f"{path}{class_name}_{filename}.png")
-        else:
-            shap.summary_plot(shap_values, X_test_sample)
-            plt.tight_layout()
+        shap.summary_plot(shap_values, X_test_sample)
+        plt.tight_layout()
+        plt.savefig(f"{path}_{filename}.png")
 
         return shap_values
 
+
+
     def most_common_words_by_group(
         self,
         X,
@@ -497,9 +508,25 @@ class ModelResults:
                 print("\n", file=f)
 
                 # Precision / recall / f1-score for each predicted class
+                report_df = (
+                    pd.DataFrame(
+                        classification_report(
+                            y_test,
+                            y_predicted,
+                            target_names=y_test.columns,
+                            output_dict=True,
+                        )
+                    )
+                    .transpose()
+                    .drop(["micro avg", "macro avg", "weighted avg", "samples avg"])
+                    .drop("support", axis=1)
+                )
                 print(
-                    classification_report(
-                        y_test, y_predicted, target_names=y_test.columns
+                    tabulate(
+                        report_df,
+                        headers="keys",
+                        tablefmt="pipe",
+                        numalign="left",
                     ),
                     file=f,
                 )
@@ -519,7 +546,16 @@ class ModelResults:
                 # Print separately so class name gets printed cleanly first
                 for i in range(len(y_test.columns)):
                     print(y_test.columns[i], file=f)
-                    print(matrix_dfs[i], file=f)
+                    print(
+                        tabulate(
+                            matrix_dfs[i],
+                            headers="keys",
+                            tablefmt="pipe",
+                            numalign="left",
+                        ),
+                        file=f,
+                    )
+                    print("\n", file=f)
 
         elif model_type.lower() == "classification":
             pscore = round(precision_score(y_test, y_predicted), 3)
@@ -537,20 +573,36 @@ class ModelResults:
                 crosstab = pd.crosstab(
                     np.array(y_test),
                     y_predicted,
-                    rownames=["Actual"],
-                    colnames=["Predicted"],
                 )
-                print(crosstab, file=f)
+                class_values = crosstab.columns
+                crosstab.columns = [f"Predicted {val}" for val in class_values]
+                crosstab.index = [f"Actual {val}" for val in class_values]
+                print(
+                    tabulate(
+                        crosstab,
+                        headers="keys",
+                        tablefmt="pipe",
+                        numalign="left",
+                    ),
+                    file=f,
+                )
                 print("\n", file=f)
 
                 sum_crosstab = crosstab.to_numpy().sum()
+                prop_crosstab = pd.crosstab(
+                    np.array(y_test),
+                    y_predicted,
+                ).apply(lambda r: round(r / sum_crosstab, 3))
+                class_values = prop_crosstab.columns
+                prop_crosstab.columns = [f"Predicted {val}" for val in class_values]
+                prop_crosstab.index = [f"Actual {val}" for val in class_values]
                 print(
-                    pd.crosstab(
-                        np.array(y_test),
-                        y_predicted,
-                        rownames=["Actual"],
-                        colnames=["Predicted"],
-                    ).apply(lambda r: round(r / sum_crosstab, 3)),
+                    tabulate(
+                        prop_crosstab,
+                        headers="keys",
+                        tablefmt="pipe",
+                        numalign="left",
+                    ),
                     file=f,
                 )
         else:
